@@ -37,10 +37,7 @@ HANDLE hAFSDMainThread = NULL;
 HANDLE hAFSDWorkerThread[WORKER_THREADS];
 #endif
 
-/* for the IFS version, set the event DoTerminate, on which all
-   worker threads wait.  they will exit, and then everything else
-   can uninitialize. */
-HANDLE WaitToTerminate, DoTerminate;
+HANDLE WaitToTerminate;
 
 int GlobalStatus;
 
@@ -59,6 +56,10 @@ int powerEventsRegistered = 0;
  */
 static void afsd_notifier(char *msgp, char *filep, long line)
 {
+#ifdef AFSIFS
+    int i;
+#endif
+
     if (filep)
     	LogEvent(EVENTLOG_ERROR_TYPE, MSG_SERVICE_ERROR_STOP_WITH_MSG_AND_LOCATION, 
                  filep, line, msgp);
@@ -85,10 +86,8 @@ static void afsd_notifier(char *msgp, char *filep, long line)
     DebugBreak();	
 #endif
 
-#ifndef AFSIFS
     SetEvent(WaitToTerminate);
-#else
-    SetEvent(DoTerminate);
+#ifdef AFSIFS
     WaitForMultipleObjects(WORKER_THREADS, hAFSDWorkerThread, TRUE, INFINITE);
     for (i = 0; i < WORKER_THREADS; i++)
         CloseHandle(hAFSDWorkerThread[i]);
@@ -197,11 +196,7 @@ afsd_ServiceControlHandler(DWORD ctrlCode)
         }
 
       doneTrace:
-#ifndef AFSIFS
         SetEvent(WaitToTerminate);
-#else
-        SetEvent(DoTerminate);
-#endif
         break;
 
     case SERVICE_CONTROL_INTERROGATE:
@@ -269,11 +264,7 @@ afsd_ServiceControlHandlerEx(
         }
 
       doneTrace:
-#ifndef AFSIFS
         SetEvent(WaitToTerminate);
-#else
-        SetEvent(DoTerminate);
-#endif
         dwRet = NO_ERROR;
         break;
 
@@ -381,10 +372,13 @@ afsd_ServiceControlHandlerEx(
  * Mount a drive into AFS if there global mapping
  */
 /* DEE Could check first if we are run as SYSTEM */
-#define MAX_RETRIES 30
-static void MountGlobalDrives(void)
+#define MAX_RETRIES 10
+#define MAX_DRIVES  23
+static DWORD __stdcall MountGlobalDrivesThread(void * notUsed)
 {
+#ifndef AFSIFS
     char szAfsPath[_MAX_PATH];
+#endif
     char szDriveToMapTo[5];
     DWORD dwResult;
     char szKeyName[256];
@@ -401,7 +395,7 @@ static void MountGlobalDrives(void)
     if (dwResult != ERROR_SUCCESS)
         return;
 
-    while (dwRetry < MAX_RETRIES) {
+    while (dwIndex < MAX_DRIVES) {
         dwDriveSize = sizeof(szDriveToMapTo);
         dwSubMountSize = sizeof(szSubMount);
         dwResult = RegEnumValue(hKey, dwIndex++, szDriveToMapTo, &dwDriveSize, 0, &dwType, szSubMount, &dwSubMountSize);
@@ -414,7 +408,7 @@ static void MountGlobalDrives(void)
         }
 
 #ifndef AFSIFS
-        for ( ; dwRetry < MAX_RETRIES; dwRetry++)
+        for (dwRetry = 0 ; dwRetry < MAX_RETRIES; dwRetry++)
         {
             NETRESOURCE nr;
             memset (&nr, 0x00, sizeof(NETRESOURCE));
@@ -442,24 +436,61 @@ static void MountGlobalDrives(void)
         }
 #else
 	/* FIXFIX: implement */
+	afsi_log("GlobalAutoMap of %s to %s not implemented", szDriveToMapTo, szSubMount);
 #endif
     }        
 
     RegCloseKey(hKey);
+    return 0;
+}
+
+static HANDLE hThreadMountGlobalDrives = NULL;
+
+static void MountGlobalDrives()
+{
+    DWORD tid;
+
+    hThreadMountGlobalDrives = CreateThread(NULL, 0, MountGlobalDrivesThread, 0, 0, &tid);
+
+    if ( hThreadMountGlobalDrives ) {
+        DWORD rc = WaitForSingleObject( hThreadMountGlobalDrives, 15000 );
+	if (rc == WAIT_TIMEOUT) {
+	    afsi_log("GlobalAutoMap thread failed to complete after 15 seconds");
+	} else if (rc == WAIT_OBJECT_0) {
+	    afsi_log("GlobalAutoMap thread completed");
+	    CloseHandle( hThreadMountGlobalDrives );
+	    hThreadMountGlobalDrives = NULL;
+	}
+    }
 }
 
 static void DismountGlobalDrives()
 {
+#ifndef AFSIFS
     char szAfsPath[_MAX_PATH];
     char szDriveToMapTo[5];
-    DWORD dwResult;
-    char szKeyName[256];
-    HKEY hKey;
-    DWORD dwIndex = 0;
     DWORD dwDriveSize;
     DWORD dwSubMountSize;
     char szSubMount[256];
     DWORD dwType;
+#endif
+    DWORD dwResult;
+    char szKeyName[256];
+    HKEY hKey;
+    DWORD dwIndex = 0;
+
+    if ( hThreadMountGlobalDrives ) {
+        DWORD rc = WaitForSingleObject(hThreadMountGlobalDrives, 0);
+
+	if (rc == WAIT_TIMEOUT) {
+	    afsi_log("GlobalAutoMap thread failed to complete before service shutdown");
+	}
+	else if (rc == WAIT_OBJECT_0) {
+	    afsi_log("GlobalAutoMap thread completed");
+	    CloseHandle( hThreadMountGlobalDrives );
+	    hThreadMountGlobalDrives = NULL;
+	}
+    }
 
     sprintf(szKeyName, "%s\\GlobalAutoMapper", AFSREG_CLT_SVC_PARAM_SUBKEY);
 
@@ -467,8 +498,10 @@ static void DismountGlobalDrives()
     if (dwResult != ERROR_SUCCESS)
         return;
 
-#ifndef AFSIFS    
-    while (1) {
+#ifdef AFSIFS    
+    /* FIXFIX: implement */
+#else
+    while (dwIndex < MAX_DRIVES) {
         dwDriveSize = sizeof(szDriveToMapTo);
         dwSubMountSize = sizeof(szSubMount);
         dwResult = RegEnumValue(hKey, dwIndex++, szDriveToMapTo, &dwDriveSize, 0, &dwType, szSubMount, &dwSubMountSize);
@@ -487,8 +520,6 @@ static void DismountGlobalDrives()
         
         afsi_log("Disconnect from GlobalAutoMap of %s to %s %s", szDriveToMapTo, szSubMount, dwResult ? "succeeded" : "failed");
     }        
-#else
-	/* FIXFIX: implement */
 #endif
 
     RegCloseKey(hKey);
@@ -1033,6 +1064,9 @@ afsd_Main(DWORD argc, LPTSTR *argv)
 #endif /* JUMP */
     HMODULE hHookDll;
     HMODULE hAdvApi32;
+#ifdef AFSIFS
+    int cnt;
+#endif
 
 #ifdef _DEBUG
     _CrtSetDbgFlag(_CRTDBG_ALLOC_MEM_DF /*| _CRTDBG_CHECK_ALWAYS_DF*/ | 
@@ -1050,12 +1084,6 @@ afsd_Main(DWORD argc, LPTSTR *argv)
     if ( GetLastError() == ERROR_ALREADY_EXISTS )
         afsi_log("Event Object Already Exists: %s", TEXT("afsd_service_WaitToTerminate"));
 
-#ifdef AFSIFS
-    DoTerminate = CreateEvent(NULL, TRUE, FALSE, TEXT("afsd_service_DoTerminate"));
-    if ( GetLastError() == ERROR_ALREADY_EXISTS )
-        afsi_log("Event Object Already Exists: %s", TEXT("afsd_service_DoTerminate"));
-#endif
-
 #ifndef NOTSERVICE
     hAdvApi32 = LoadLibrary("advapi32.dll");
     if (hAdvApi32 == NULL)
@@ -1467,11 +1495,9 @@ main(int argc, char * argv[])
 		
             printf("Hit <Enter> to terminate OpenAFS Client Service\n");
             getchar();  
-#ifndef AFSIFS
             SetEvent(WaitToTerminate);
-#else
-            SetEvent(DoTerminate);
-			dc_release_hooks();
+#ifdef AFSIFS
+	    dc_release_hooks();
 #endif
         }
     }
