@@ -16,7 +16,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_util.c,v 1.28 2006/04/26 15:03:20 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/afs_util.c,v 1.28.2.3 2006/08/02 19:01:22 shadow Exp $");
 
 #include "afs/stds.h"
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
@@ -51,10 +51,6 @@ RCSID
 #include <sys/fp_io.h>
 #endif
 
-#if	defined(AFS_XBSD_ENV)
-#include <crypto/md5.h>
-#endif
-
 afs_int32 afs_new_inum = 0;
 
 #ifndef afs_cv2string
@@ -409,14 +405,7 @@ afs_data_pointer_to_int32(const void *p)
     return ip.i32[i32_sub];
 }
 
-#if	defined(AFS_XBSD_ENV)
-
-#define afs_md5 MD5Context
-#define AFS_MD5_Init(m) MD5Init((m))
-#define AFS_MD5_Update(m, p, l) MD5Update((m), (void *)(p), (l))
-#define AFS_MD5_Final(r, m) MD5Final((r), (m))
-
-#else
+#ifdef AFS_LINUX20_ENV
 
 struct afs_md5 {
     unsigned int sz[2];
@@ -572,7 +561,7 @@ calc (struct afs_md5 *m, afs_uint32 *data)
  * From `Performance analysis of MD5' by Joseph D. Touch <touch@isi.edu>
  */
 
-#if defined(AFSBIG_ENDIAN)
+#if defined(WORDS_BIGENDIAN)
 static inline afs_uint32
 swap_u_int32_t (afs_uint32 t)
 {
@@ -610,15 +599,15 @@ AFS_MD5_Update (struct afs_md5 *m, const void *v, size_t len)
 	p += l;
 	len -= l;
 	if(offset == 64){
-#if defined(AFSBIG_ENDIAN)
+#if defined(WORDS_BIGENDIAN)
 	    int i;
-	    afs_uint32 current[16];
+	    afs_uint32 temp[16];
 	    struct x32 *us = (struct x32*)m->save;
 	    for(i = 0; i < 8; i++){
-		current[2*i+0] = swap_u_int32_t(us[i].a);
-		current[2*i+1] = swap_u_int32_t(us[i].b);
+		temp[2*i+0] = swap_u_int32_t(us[i].a);
+		temp[2*i+1] = swap_u_int32_t(us[i].b);
 	    }
-	    calc(m, current);
+	    calc(m, temp);
 #else
 	    calc(m, (afs_uint32*)m->save);
 #endif
@@ -657,7 +646,6 @@ AFS_MD5_Final (void *res, struct afs_md5 *m)
 	}
     }
 }
-#endif
 
 afs_int32 afs_calc_inum (afs_int32 volume, afs_int32 vnode)
 { 
@@ -677,3 +665,12 @@ afs_int32 afs_calc_inum (afs_int32 volume, afs_int32 vnode)
     }
     return ino;
 }
+
+#else
+
+afs_int32 afs_calc_inum (afs_int32 volume, afs_int32 vnode)
+{
+    return (volume << 16) + vnode;
+}
+
+#endif
