@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/ubik/ubik.c,v 1.14 2003/07/15 23:17:05 shadow Exp $");
+    ("$Header: /cvs/openafs/src/ubik/ubik.c,v 1.15.14.1 2006/06/12 21:56:24 shadow Exp $");
 
 #include <sys/types.h>
 #ifdef AFS_NT40_ENV
@@ -185,31 +185,6 @@ ContactQuorum(aproc, atrans, aflags, aparm0, aparm1, aparm2, aparm3, aparm4,
     Note that the host named by myHost should not also be listed in serverList.
 */
 
-int
-ubik_ServerInitByInfo(afs_int32 myHost, short myPort,
-		      struct afsconf_cell *info, char clones[],
-		      char *pathName, struct ubik_dbase **dbase)
-{
-    afs_int32 code;
-
-    code =
-	ubik_ServerInitCommon(myHost, myPort, info, clones, 0, pathName,
-			      dbase);
-    return code;
-}
-
-int
-ubik_ServerInit(afs_int32 myHost, short myPort, afs_int32 serverList[],
-		char *pathName, struct ubik_dbase **dbase)
-{
-    afs_int32 code;
-
-    code =
-	ubik_ServerInitCommon(myHost, myPort, (struct afsconf_cell *)0, 0,
-			      serverList, pathName, dbase);
-    return code;
-}
-
 int
 ubik_ServerInitCommon(afs_int32 myHost, short myPort,
 		      struct afsconf_cell *info, char clones[],
@@ -265,6 +240,8 @@ ubik_ServerInitCommon(afs_int32 myHost, short myPort,
 	    ubik_sc[secIndex] = secClass;
 	}
     }
+    /* for backwards compat this should keep working as it does now 
+       and not host bind */
     code = rx_Init(myPort);
     if (code < 0)
 	return code;
@@ -321,6 +298,31 @@ ubik_ServerInitCommon(afs_int32 myHost, short myPort,
     return code;
 }
 
+int
+ubik_ServerInitByInfo(afs_int32 myHost, short myPort,
+		      struct afsconf_cell *info, char clones[],
+		      char *pathName, struct ubik_dbase **dbase)
+{
+    afs_int32 code;
+
+    code =
+	ubik_ServerInitCommon(myHost, myPort, info, clones, 0, pathName,
+			      dbase);
+    return code;
+}
+
+int
+ubik_ServerInit(afs_int32 myHost, short myPort, afs_int32 serverList[],
+		char *pathName, struct ubik_dbase **dbase)
+{
+    afs_int32 code;
+
+    code =
+	ubik_ServerInitCommon(myHost, myPort, (struct afsconf_cell *)0, 0,
+			      serverList, pathName, dbase);
+    return code;
+}
+
 /*  This routine begins a read or write transaction on the transaction
     identified by transPtr, in the dbase named by dbase.  An open mode of
     ubik_READTRANS identifies this as a read transaction, while a mode of
